#include <windows.h>
#include <commctrl.h>
#include "mixvibesdsp.h"

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    return TRUE;
}

/*// avoid stupid CRT silliness
BOOL WINAPI _DllMainCRTStartup(HANDLE hInst, ULONG ul_reason_for_call, LPVOID lpReserved)
{
	return TRUE;
}
*/

// module getter.
mixvibesDSPModule *getModule(void);

// Module header, includes version, description, and address of the module retriever function
mixvibesDSPHeader hdr = { 1, "MixVibes GARGLE",  "InVibes", getModule};

#ifdef __cplusplus
extern "C" 
{
#endif
// this is the only exported symbol. returns our main header.
__declspec( dllexport ) mixvibesDSPHeader *mixvibesDSPGetHeader()
{
	return &hdr;
}
#ifdef __cplusplus
}
#endif

// param definition
#define NBPARAM			3

#define	PARAMVOLWET		0
#define	PARAMVOLDRY		1
#define	PARAMFREQ		2

typedef struct Gargle
{
	int		ind;
	int		period;
	int     sampling; 
	int		nbchannel;
	float	bpm;
	MVDSP_PARAM_VALUE  valNum[NBPARAM];
	BOOL start;
} Gargle;

// export functions for mixvibes 
void Info(HWND hwnd);
mixvibesDSPModule *Init(int sampling, int nbchannel, float bpm);
void SetStart (mixvibesDSPModule *this_mod,  BOOL start, MVDSP_PARAM_VALUE *valNum);
int Process(mixvibesDSPModule *this_mod, 
		float *samples,
		int numsamples,
		int position);
void OnChange (mixvibesDSPModule *this_mod,
			   unsigned int indiceTab, 
			   MVDSP_PARAM_VALUE value);

void UpdateBpm (struct mixvibesDSPModule *this_mod, float bpm);

BOOL PadProcess(struct mixvibesDSPModule *this_mod, float x, float y, float intensity);

void Quit (mixvibesDSPModule *this_mod);

mixvibesParam tabParam[NBPARAM]=
{
	{TYPEPARAMREAL1,MVDSP_PARAM_NO_FLAG,"Wet/Dry mix",0.0f,0.0f,1.0f},	// param 0: volume (wet) set by mixvibes
										// you can init a default volume
	{TYPEPARAMREAL1,MVDSP_PARAM_NO_FLAG,"Wet/Dry mix",0.0f,0.0f,0.0f},// param 1: dry level=(1-volume) always set by mixvibes
									// let NULL initialized

									// effects specific parameters begins at indice 2
	{TYPEPARAMINT, MVDSP_PARAM_NO_FLAG, "Freq",5.0f, 1.0f, 50.0f},	// param 3: freq
};

mixvibesDSPModule mod= 
{
  NULL,							// parent window 
  NULL,							// instance handle to this DLL 
  MIXVIBES_DSP_SDK_VERSION,  	// mixvibes DSP-SDK version 
  MVDSP_CANDO_BLOCK_PROCESS|MVDSP_CANDO_SAMPLE_PROCESS|MVDSP_CANDO_WET_DRY_MIX,   	    // module flags             
  1,							// mono by default
  0,							// no sampling rate set when init
  0.0f, 						// default tempo (BPM)
  FALSE,						// DSP status = off when init  
  NULL,							// hBitmap
  NBPARAM,	    // nbParam
  tabParam,						// tabParam
  NULL,							// values are created on init
  NULL,                         // no user data for now
  Info,
  Init,
  SetStart,
  Process,
  NULL,
  Quit,
  OnChange,
  NULL,
  NULL,
  NULL
};

// getmodule routine from the main header. Returns NULL if an invalid module was requested,
// otherwise returns either mod1 or mod2 depending on 'which'.
mixvibesDSPModule *getModule(void)
{
	return &mod;
}

void Info(HWND hwnd)
{
	MessageBox(hwnd,"MixVibes Gargle Demo 1.0","MixVibes DSP",MB_OK);
}

mixvibesDSPModule *Init(int sampling, int nbchannel, float bpm)
{
	Gargle *puserData;
	mixvibesDSPModule *newmod= malloc(sizeof(mixvibesDSPModule));
	if (newmod == NULL)
		return NULL;

	memcpy(newmod, &mod, sizeof(*newmod));

	puserData= malloc(sizeof(Gargle)); 
	
	if (puserData == NULL)
		return NULL;

	memset(puserData, 0, sizeof(*puserData));

	puserData->sampling= sampling;
	puserData->nbchannel= nbchannel;
	puserData->bpm= bpm;

	newmod->puserData= puserData;

	return newmod;
}

void SetStart (mixvibesDSPModule *this_mod, BOOL start, MVDSP_PARAM_VALUE *valNum)
{
	Gargle *puserData;
	if (this_mod == NULL || this_mod->puserData == NULL)
		return;

	puserData= (Gargle *)(this_mod->puserData); 

	if (start == TRUE) {
		int i;
		if (valNum[PARAMFREQ]==0) // default value
			valNum[PARAMFREQ]=5;

		for (i=0; i < NBPARAM; i++)
			puserData->valNum[i]= valNum[i];

		puserData->ind= 0;
		puserData->period= puserData->sampling / (int)puserData->valNum[PARAMFREQ];
		puserData->start= TRUE;
	}
	else {
		puserData->start= FALSE;
	}
}

int Process(mixvibesDSPModule *this_mod, 
		float *samples, // samples table
		int numsamples, // number of samples 
		int position) 
{
	int nb;
	float *pEnd;
	float voldry, volwet;
	Gargle *puserData;
	double period2;

	if (this_mod == NULL || this_mod->puserData == NULL)
		return numsamples;

	puserData= (Gargle *)(this_mod->puserData); 
	if (puserData->start== FALSE)
		return numsamples;

	period2=  puserData->period / 2;

	nb= numsamples * puserData->nbchannel;

	pEnd= samples + nb;
	volwet= puserData->valNum[PARAMVOLWET];
	voldry= puserData->valNum[PARAMVOLDRY];
	
	switch (puserData->nbchannel) {
	case 1:
		for (; samples < pEnd; samples++) {
			double val;
			if (puserData->ind < period2)
				val= (double)puserData->ind /period2;
			else 
				val=1 - ( ((double)puserData->ind - period2) / period2);

			*samples= (float)((*samples * voldry) + (*samples * val * volwet));

			if (++puserData->ind > puserData->period)
				puserData->ind= 0;
		}
		break;
	case 2:
		for (; samples < pEnd; )	{
			double val;
			if (puserData->ind < period2)
				val= (double)puserData->ind /period2;
			else 
				val=1 - ( ((double)puserData->ind - period2) / period2);

			*samples= (float) ((*samples * val * volwet))+(*samples * voldry ); samples++;
			*samples= (float) ((*samples * val * volwet))+(*samples * voldry ); samples++;

			if (++puserData->ind > puserData->period)
				puserData->ind= 0;
		}
		break;
	}

	return numsamples;
}

void Quit (mixvibesDSPModule *this_mod)
{
	if (this_mod->puserData) {
		free(this_mod->puserData);
		this_mod->puserData= NULL;
	}

	free(this_mod);
}

// wParam control indice in the tabparam
// lParam new value for the param
void OnChange (mixvibesDSPModule *this_mod,
			   unsigned int indiceTab, 
			   MVDSP_PARAM_VALUE value)
{
	Gargle *puserData;
	if (this_mod == NULL ||	this_mod->puserData == NULL)
		return;

	puserData= (Gargle *)(this_mod->puserData); 
 
	switch (indiceTab) {
	case PARAMVOLWET: // volume
		puserData->valNum[PARAMVOLWET]= value;
		return;

	case PARAMVOLDRY: // volume
		puserData->valNum[PARAMVOLDRY]= value;
		return;

	case PARAMFREQ: // freq
		puserData->valNum[PARAMFREQ]= value;
		if (puserData->start== TRUE) {
			SetStart (this_mod, TRUE, puserData->valNum);
		}
		return;
	}
}

void UpdateBpm (struct mixvibesDSPModule *this_mod, float bpm)
{
}

BOOL PadProcess(struct mixvibesDSPModule *this_mod, float x, float y, float intensity)
{
	return FALSE;
}
